/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.engines;

import Jama.Matrix;
import org.objectweb.proactive.extra.montecarlo.SimulationSet;
import plugtest.data.AssetPool;
import plugtest.data.BasketParams;
import plugtest.data.Result;
import plugtest.data.TestCase;
import plugtest.data.types.AlgorithmType;
import plugtest.data.types.OptionType;
import plugtest.data.types.PayOffType;

import java.io.Serializable;


/**
 * An abstract class to define a skeleton for pricing monte carlo based option
 * pricing algorithms. The class implements the SimulationSet defined in the
 * ProActive Monte Carlo API.
 *
 * @author The ProActive Team
 */
public abstract class AbstractMCPricingAlgorithm implements Serializable, SimulationSet<Result> {

    /**
     * risk free interest rate
     */
    protected double interestRiskFreeRate;

    /**
     * time steps per year
     */
    protected double timeIntervalsPerYear;

    /**
     * required for computing theta (greeks)
     */
    protected double epsilonT; // time step

    /**
     * required for computing delta, gamma and speed (greeks)
     */
    protected double epsilonS;

    /**
     * required for computing rho (greeks)
     */
    protected double epsilonR;

    protected AssetPool testCaseAssetPool;

    protected long mcIterations; // total monte carlo iterations

    protected PayOffType payoffType;
    protected AlgorithmType algorithmType;
    protected OptionType optionType;

    /**
     * time to maturity of the basket
     */

    protected double timeToMaturity;

    /**
     * correlation matrix of the assets for the test case
     */
    protected Matrix correlationMatrix;

    protected double testCaseSrikePrice;
    protected double testCaseBarrier;

    AbstractMCPricingAlgorithm(BasketParams basketParams, TestCase testCase, AssetPool testCaseAssetPool,
            Matrix correlationMatrix, long mciter) {
        // set the basket related parameters
        this.interestRiskFreeRate = basketParams.getRiskFreeRate();
        this.timeIntervalsPerYear = basketParams.getTimeIntervalsPerYear();
        this.epsilonT = basketParams.getEpsilonT();
        this.epsilonS = basketParams.getEpsilonS();
        this.epsilonR = basketParams.getEpsilonR();

        // set the test case asset pool
        this.testCaseAssetPool = testCaseAssetPool;

        // set the correlation matrix corresponding to the asset pool
        this.correlationMatrix = correlationMatrix;

        // set the test case parameters
        this.optionType = testCase.getOption();
        this.algorithmType = testCase.getAlgorithm();
        this.payoffType = testCase.getPayOff();
        this.timeToMaturity = testCase.getTimeToMaturity();

        // set the total number monte carlo simulations per algorithm task
        this.mcIterations = mciter;

        // compute the strike price of the testcase asset pool
        this.testCaseSrikePrice = plugtest.pricing.util.AlgorithmUtil.computeStrike(testCaseAssetPool,
                payoffType);

        // At the beginning of time interval, at t = 0, the spot price is equal
        // to the strike price
        double testCaseSpotPrice = this.testCaseSrikePrice;

        this.testCaseBarrier = plugtest.pricing.util.AlgorithmUtil.computerBarrier(testCaseSpotPrice,
                testCase.getBarrier(), algorithmType);
    }

}
