/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.engines;

import Jama.Matrix;
import org.objectweb.proactive.extra.montecarlo.EngineTask;
import plugtest.data.AssetPool;
import plugtest.data.BasketParams;
import plugtest.data.Result;
import plugtest.data.TestCase;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;


/**
 * Abstract pricing engine
 *
 * @author The ProActive Team
 */
public abstract class AbstractPricingEngine implements EngineTask<Result> {

    protected AssetPool assetPool;
    protected BasketParams basketParams;
    protected Matrix correlationMatrix;
    protected TestCase testCase;

    protected URL descriptor;
    public static final String DEFAULT_DESCRIPTOR = "WorkersApplication.xml";
    public static final String DEFAULT_WORKERS_NAME = "Workers";

    /**
     * The number of task can be set depending on the number of computing nodes
     * available.
     */

    public static final int DEFAULT_NUMBER_TASKS = 100;

    /**
     * The number of monte carlo iterations per task can be tweaked for
     * performance and accuracy. More the number of iterations, the better will
     * be the convergence though might cause increase in the computational time.
     * You can devise your own strategy for setting the number of monte carlo
     * simulations as well as the number of tasks for different test cases.
     */
    public static final long DEFAULT_MCITER = 1000;

    protected static int niter = 0;
    protected static int tasks = 0;
    protected static URL descriptor_url;
    protected String vn_name;
    protected String master_vn_name;

    public AbstractPricingEngine() {
        super();
    }

    public AbstractPricingEngine(File descrFile) {
        super();

        if (!descrFile.exists()) {
            System.err.println("" + descrFile + " does not exist.");
            System.exit(1);
        } else if (!descrFile.canRead()) {
            System.err.println("" + descrFile + " can not be read.");
            System.exit(1);
        } else if (!descrFile.isFile()) {
            System.err.println("" + descrFile + " is not a regular file.");
            System.exit(1);
        }
        try {
            descriptor = descrFile.toURI().toURL();
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }

        vn_name = DEFAULT_WORKERS_NAME;
        master_vn_name = null;
    }

    /**
     * Sets the deployment descriptor file, the virtual node name and the master
     * virtual node name for the engine
     *
     * @param descrFile      url of the descriptor file
     * @param vn_name        virtual node name
     * @param master_vn_name name of the master virtual node
     */
    public AbstractPricingEngine(File descrFile, String vn_name, String master_vn_name) {
        super();
        if (!descrFile.exists()) {
            System.err.println("" + descrFile + " does not exist.");
            System.exit(1);
        } else if (!descrFile.canRead()) {
            System.err.println("" + descrFile + " can not be read.");
            System.exit(1);
        } else if (!descrFile.isFile()) {
            System.err.println("" + descrFile + " is not a regular file.");
            System.exit(1);
        }

        try {
            descriptor = descrFile.toURI().toURL();
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }

        if (vn_name != null)
            this.vn_name = vn_name;
        else
            this.vn_name = DEFAULT_WORKERS_NAME;
        this.master_vn_name = master_vn_name;
    }

    public URL getDescriptor() {
        return descriptor;
    }

    public void setDescriptor(URL descriptor) {
        this.descriptor = descriptor;
    }

    public String getVn_name() {
        return vn_name;
    }

    public void setVn_name(String vn_name) {
        this.vn_name = vn_name;
    }

    public BasketParams getBasketParams() {
        return basketParams;
    }

    public void setBasketParams(BasketParams basketParams) {
        this.basketParams = basketParams;
    }

    public Matrix getCorrelationMatrix() {
        return correlationMatrix;
    }

    public void setCorrelationMatrix(Matrix correlationMatrix) {
        this.correlationMatrix = correlationMatrix;
    }

    public AssetPool getAssetPool() {
        return assetPool;
    }

    public void setAssetPool(AssetPool assetPool) {
        this.assetPool = assetPool;
    }

    public TestCase getTestCase() {
        return testCase;
    }

    public void setTestCase(TestCase testCase) {
        this.testCase = testCase;
    }

    public String getMaster_vn_name() {
        return master_vn_name;
    }

    public void setMaster_vn_name(String master_vn_name) {
        this.master_vn_name = master_vn_name;
    }

}
