/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.engines;

import java.util.ArrayList;
import java.util.Arrays;

import Jama.Matrix;
import plugtest.data.*;
import plugtest.data.types.AlgorithmType;
import plugtest.data.types.OptionType;
import plugtest.pricing.util.AlgorithmUtil;
import plugtest.pricing.util.MatrixUtil;
import plugtest.pricing.util.PayOff;
import umontreal.iro.lecuyer.probdist.NormalDist;
import umontreal.iro.lecuyer.randvar.NormalGen;
import umontreal.iro.lecuyer.rng.RandomStream;


/**
 * This is a core monte carlo based algorithm for pricing european vanilla and
 * barrier options.
 *
 * @author The ProActive Team
 */
public class MCEuropeanPricingAlgorithm extends AbstractMCPricingAlgorithm {

    private static final long serialVersionUID = 1L;

    private boolean stop;
    private boolean cont;
    private PayOff payoff; // for call and put

    private double deltaT;
    private long timeIntervals;

    private Matrix lowerTriangularMatrix;

    //@snippet-start mc_european_algorithm_constructor
    /**
     * The constructor to create the instance of monte carlo based algorithm's task. 
     * Such tasks are launched by the engine.  
     * 
     * @param basketParams	the market constants, as provided in the testcases 
     * @param testCase		individual test case
     * @param testCaseAssetPool	the basket for the testcase
     * @param correlationMatrix	the correlation matrix for the assets in the basket
     * @param mciter		the number of monte carlo simulation per task, can be controlled for accurary and speed
     */
    public MCEuropeanPricingAlgorithm(BasketParams basketParams, TestCase testCase,
            AssetPool testCaseAssetPool, Matrix correlationMatrix, long mciter)
    //@snippet-end mc_european_algorithm_constructor
    {

        super(basketParams, testCase, testCaseAssetPool, correlationMatrix, mciter);
        deltaT = (double) ((double) 1.0 / timeIntervalsPerYear);
        timeIntervals = (long) (timeIntervalsPerYear * timeToMaturity);
        lowerTriangularMatrix = MatrixUtil
                .decomposition(testCaseAssetPool.getAssetCount(), correlationMatrix);
    }

    public Result simulate(RandomStream rng) {

        double sum = 0.0;
        double sum_payoff_square = 0.0;

        double payoff = 0.0;

        double seed;
        int cnt;
        int assetCount = testCaseAssetPool.getAssetCount();

        double s_current[] = new double[assetCount];

        Matrix random = new Matrix(assetCount, 1);

        for (int d = 0; d < assetCount; d++) {
            s_current[d] = testCaseAssetPool.getAsset(d).getStartPrice();
        }

        this.payoff = new PayOff();
        java.util.Arrays.sort(s_current);
        this.payoff.setup(optionType, payoffType, s_current, testCaseSrikePrice);
        this.payoff.calculate();
        seed = this.payoff.getPre_payoff();

        NormalGen ngen = new NormalGen(rng, new NormalDist());

        for (int i = 0; i < this.mcIterations; i++) {

            for (int d = 0; d < assetCount; d++) { // reset the spot prices
                s_current[d] = testCaseAssetPool.getAsset(d).getStartPrice();
                random.set(d, 0, ngen.nextDouble());
            }

            this.stop = false;
            this.cont = false;

            cnt = 0;
            for (long j = 0; j < this.timeIntervals && this.stop == false; j++) {
                cnt++;
                random = this.lowerTriangularMatrix.times(random);
                // simulate the future prices of the assets in the basket
                for (int d = 0; d < assetCount; d++) {
                    s_current[d] = AlgorithmUtil.simulateLognormal(s_current[d], this.interestRiskFreeRate,
                            this.testCaseAssetPool.getAsset(d).getVolatility(), this.testCaseAssetPool
                                    .getAsset(d).getDividend(), this.deltaT, random.get(d, 0));
                    random.set(d, 0, ngen.nextDouble());
                }
                java.util.Arrays.sort(s_current);
                barrierTesting(seed, s_current);
            }
            if (this.stop == true && this.cont == true) {
                // In case of barrier IN options
                for (int k = cnt; k < this.timeIntervals; k++) {
                    random = this.lowerTriangularMatrix.times(random);
                    for (int d = 0; d < assetCount; d++) {
                        s_current[d] = AlgorithmUtil.simulateLognormal(s_current[d],
                                this.interestRiskFreeRate,
                                this.testCaseAssetPool.getAsset(d).getVolatility(), this.testCaseAssetPool
                                        .getAsset(d).getDividend(), this.deltaT, random.get(d, 0));
                        random.set(d, 0, ngen.nextDouble());
                    }
                }

                java.util.Arrays.sort(s_current);
                this.payoff.setup(optionType, payoffType, s_current, testCaseSrikePrice);
                this.payoff.calculate();

                payoff = discountPayoff(this.payoff.getPayoff());

            }
            if (this.stop == true && this.cont == false) {
                // in case of barrier OUT options
                // THE PARTICIPANTS WILL HAVE TO IMPLEMENT THE SNIPPET
                payoff = 0;
            }
            if (this.stop == false && this.cont == false) {
                // in case of vanilla options
                payoff = discountPayoff(this.payoff.getPayoff());
            }

            sum += payoff;
            sum_payoff_square += (payoff * payoff);
        }

        Result pResult = new Result();

        Price price = new Price();

        if (optionType.getType() == OptionType.CALL_TYPE)
            price.setCallPrice(sum);
        else
            price.setPutPrice(sum);
        price.setVariance(sum_payoff_square);

        /* fillers : The participants will have to compute the values */
        price.setCI(0.0);

        Greeks greeks = new Greeks();
        double fillerArray[] = new double[assetCount];
        for (int i = 0; i < assetCount; i++)
            fillerArray[i] = 0.0;
        greeks.setDelta(fillerArray);
        greeks.setGamma(fillerArray);
        greeks.setRho(0.0);
        greeks.setSpeed(fillerArray);
        greeks.setTheta(0.0);

        pResult.setPrice(price);
        pResult.setGreeks(greeks);

        return pResult;
    }

    /**
     * Discounting the approximation of future price by discount factor
     * (e^-(r*T)) we get an approximation of the present day fair derivative
     * price.
     *
     * @param payoff the payoff of the option for a path
     * @return discounted payoff
     */
    private double discountPayoff(double payoff) {

        return payoff * Math.exp(-this.interestRiskFreeRate * this.timeToMaturity);

    }

    /**
     * Tests whether the current payoff of the basket has hit or passed the
     * barrier.
     *
     * @param seed
     * @param s_current_sorted current values of the spot prices.
     */

    public void barrierTesting(double seed, double[] s_current_sorted) {

        this.payoff.setup(optionType, payoffType, s_current_sorted, testCaseSrikePrice);
        this.payoff.calculate();

        if (algorithmType.getType() != 1) {
            if (testCaseBarrier >= seed) { // UP barrier
                if (algorithmType.getType() == AlgorithmType.BARRIERUPIN_TYPE) {

                    if (this.payoff.getPre_payoff() >= testCaseBarrier) {
                        // the pre payoff has hit the barrier
                        // activate the option
                        this.stop = true;
                        this.cont = true;

                    } else {
                        // the pre pay off is still under the barrier
                        // do not activate the option
                        this.payoff.Reset();

                    }
                }
                if (algorithmType.getType() == AlgorithmType.BARRIERUPOUT_TYPE) { // OUT
                    if (this.payoff.getPre_payoff() >= testCaseBarrier) {
                        // pre payoff has hit the barrier
                        // the payoff will be invalid ( out type option)
                        this.stop = true;
                        this.cont = false;
                        this.payoff.Reset();
                    }
                }

            } else { // DOWN barrier
                if (algorithmType.getType() == AlgorithmType.BARRIERDOWNIN_TYPE) { // IN
                    if (this.payoff.getPre_payoff() <= testCaseBarrier) {
                        // the pre pay off has passed the lower barrier
                        // activate the option
                        this.stop = true;
                        this.cont = true;
                    } else {
                        // the pre pay off is still above the lower barrier
                        // do not activate the option yet
                        this.payoff.Reset();
                    }
                }
                if (algorithmType.getType() == AlgorithmType.BARRIERDOWNOUT_TYPE) { // OUT
                    if (this.payoff.getPre_payoff() <= testCaseBarrier) {
                        // the pre pay passed the lower barrier
                        // deactive the option (out type option)
                        this.stop = true;
                        this.cont = false;
                        this.payoff.Reset();
                    }
                }
            }
        }
    }

}
