/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.engines;

import org.objectweb.proactive.extensions.masterworker.TaskException;
import org.objectweb.proactive.extra.montecarlo.Executor;
import org.objectweb.proactive.extra.montecarlo.SimulationSet;
import org.objectweb.proactive.extra.montecarlo.Simulator;
import plugtest.data.Result;
import plugtest.pricing.util.ResultHelper;

import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;


/**
 * This class launches the bag of tasks of the monte carlo simulation for
 * computing the price of a european option. The results from the individiual
 * bags are collected and merged fot computing the final price of the option.
 *
 * @author The ProActive Team
 */

public class MCEuropeanPricingEngine extends AbstractPricingEngine {

    private static final long serialVersionUID = 1L;

    public MCEuropeanPricingEngine() {
    }

    //@snippet-start mc_european_engine_constructor
    /**
     * Constructor for the monte carlo based european option pricing engine 
     * 
     * @param descrFile	the deployment descriptor so that engine can launch the algorithm tasks.
     * @param vn_name	the virtual node name of the worker
     * @param master_vn_name	the name of virtual master node
     */
    public MCEuropeanPricingEngine(final File descrFile, String vn_name, String master_vn_name)
    //@snippet-end mc_european_engine_constructor 
    {
        super(descrFile, vn_name, master_vn_name);
    }

    public MCEuropeanPricingEngine(final File descrFile) {
        super(descrFile);
    }

    /**
     * Launches the bags of monte carlo simulations.
     */

    public Result run(final Simulator simulator, final Executor executor) {

        final List<SimulationSet<Result>> sets = new ArrayList<SimulationSet<Result>>(DEFAULT_NUMBER_TASKS);

        Enumeration<Result> simulatedPriceList = null;

        for (long i = 0; i < DEFAULT_NUMBER_TASKS; i++) {
            sets.add(new MCEuropeanPricingAlgorithm(basketParams, testCase, assetPool, correlationMatrix,
                DEFAULT_MCITER));
        }

        // Submitting these simulation sets
        try {
            simulatedPriceList = simulator.solve(sets);

        } catch (final TaskException e) {
            throw new RuntimeException(e);
        }

        final ResultHelper finalResultHelper = new ResultHelper();
        // collect individual results from workers
        while (simulatedPriceList.hasMoreElements()) {

            final Result partialResults = simulatedPriceList.nextElement();
            // accumulate all the results
            finalResultHelper.update(partialResults);
        }

        // finialize the accumulated prices. This requires the total number of
        // simulations for the algorithm.
        finalResultHelper.finalizePrices(DEFAULT_NUMBER_TASKS, DEFAULT_MCITER, testCase.getOption());

        return finalResultHelper.getResult();
    }

}