/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.option;

import Jama.Matrix;

import org.objectweb.proactive.api.PALifeCycle;
import org.objectweb.proactive.core.ProActiveException;
import org.objectweb.proactive.extensions.masterworker.TaskException;
import org.objectweb.proactive.extra.montecarlo.PAMonteCarlo;
import plugtest.data.AssetPool;
import plugtest.data.BasketParams;
import plugtest.data.Result;
import plugtest.data.TestCase;
import plugtest.pricing.engines.MCEuropeanPricingEngine;


/**
 * The class stands for an european option which has monte carlo based option
 * pricing engine.
 *
 * @author The ProActive Team
 */
public class MCEuropeanBasketOption extends AbstractBasketOption {

    private static final long serialVersionUID = 1L;
    private PAMonteCarlo<Result> mc;

    //@snippet-start mc_european_basket_constructor
    /**
     * 	The constructor takes the market constants in the basketParams, the correlation matrix for the assets in the
     *  basket in the corMatrix, the test case parameters and the asset pool in the basket
     */
    public MCEuropeanBasketOption(BasketParams basketParams, Matrix corMatrix, TestCase testCase,
            AssetPool subAssetPool)
    //@snippet-end mc_european_basket_constructor

    {
        super(basketParams, corMatrix, testCase, subAssetPool);
    }

    public MCEuropeanBasketOption(BasketParams basketParams) {
        super(basketParams);
    }

    public void set(Matrix testCaseCorrMatrix, TestCase testCase, AssetPool testCaseAssetPool) {
        this.correlationMatrix = testCaseCorrMatrix;
        this.testCase = testCase;
        this.subAssetPool = testCaseAssetPool;
        // also update the parameters associated with the engine
        engine.setAssetPool(this.subAssetPool);
        engine.setCorrelationMatrix(this.correlationMatrix);
        engine.setTestCase(this.testCase);
    }

    //@snippet-start mc_european_basket_set_engine
    /**
     * Sets the pricing engine to the option
     *
     * @param mcEuroEngine monte carlo based pricing engine to price european options
     */
    public void setPricingEngine(final MCEuropeanPricingEngine mcEuroEngine)
    //@snippet-end mc_european_basket_set_engine
    {
        engine = mcEuroEngine;
        engine.setBasketParams(this.basketParams);
    }

    /**
     * Initiate the monte carlo framework 
     */
    public void initMC() {
        try {
            mc = new PAMonteCarlo<Result>(engine.getDescriptor(), engine.getMaster_vn_name(), engine
                    .getVn_name());
        } catch (ProActiveException e) {
            System.out.println(" Exception while initiating monte carlo: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * After all the testcases, the monte carlo framework is terminated
     */
    public void terminateMC() {
        mc.terminate();
        PALifeCycle.exitSuccess();
    }

    //@snippet-start mc_european_basket_calculateMC
    /**
     * Computes price of the option using monte carlo methods.
     */
    public void calculateMC() throws ProActiveException, TaskException {

        result = mc.run(engine);
        result.setTestCaseID(engine.getTestCase().getTestCaseID());
    }
    //@snippet-end mc_european_basket_calculateMC

}
