/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.util;

import plugtest.data.AssetPool;
import plugtest.data.types.AlgorithmType;
import plugtest.data.types.PayOffType;


/**
 * Defines a set of functions, such as computing initial strike price, computing barrier or simulating a price lognormally, 
 * which needed by the pricing algorithm.
 *
 * @author The ProActive Team
 */
public class AlgorithmUtil {

    /**
     * * Computes the barrier for the algorithm based on the "barrier" parameter
     * in the test case. Note that the testCaseSpotPrice is itself derived from
     * the start prices of individual assets in the basket.
     * <p/>
     * <p/>
     * If the algorithm is of IN type the barrier value is greater than the
     * basket spot price. And if the algorithm is of OUT type the barrier value
     * is less than the basket spot price. [for details see
     * www.math.ust.hk/~maykwok/piblications/Articles/dou%20bar%20encyclo.pdf]
     * Computes the barrier relative to the spot price of the basket
     *
     * @param testCaseSpotPrice the spot price for "the basket"
     * @param barrier           the percentage barrier defined in the testcase
     * @param algorithmType     type of the vanilla/barrier algorithm
     * @return
     */

    public static double computerBarrier(double testCaseSpotPrice, double barrier, AlgorithmType algorithmType) {
        double testCaseBarrier = 0.0; // in the case of vanilla options

        switch (algorithmType.getType()) {

            case AlgorithmType.BARRIERDOWNIN_TYPE: // barrier below the current
                // price of the basket
            case AlgorithmType.BARRIERDOWNOUT_TYPE:
                /*
                 * Down barrier option consider the option with the barrier which is
                 * smaller than the spot price
                 */
                testCaseBarrier = testCaseSpotPrice - (barrier * 0.01 * testCaseSpotPrice);
                break;

            case AlgorithmType.BARRIERUPIN_TYPE: // barrier above the current
                // price of the basket
            case AlgorithmType.BARRIERUPOUT_TYPE:

                testCaseBarrier = testCaseSpotPrice + (barrier * 0.01 * testCaseSpotPrice);
                break;
        }
        return testCaseBarrier;
    }

    /**
     * * European options have been priced by using the famous Black-Scholes formula,
     * which assumes that the stock returns follow a log-normal distribution. This
     * class simulates a price of option with lognormal distribution.
     * <p/>
     * To compute the strike price for the current test case. This computed at
     * the beginning of the computation of the actual option price.
     *
     * @param testCaseAssetPool contains the assets for the current testcase.
     * @param payoffType        defines the payoff type for computing the strike price. it has
     *                          three payoff values: min, max and avg.
     * @return strikeprice returns the strike price relevant to the asset pool.
     */

    public static double computeStrike(final AssetPool testCaseAssetPool, final PayOffType payoffType) {

        double strikePrice = 0.0;

        double s_current[] = new double[testCaseAssetPool.getAssetCount()];

        for (int d = 0; d < testCaseAssetPool.getAssetCount(); d++) {
            s_current[d] = testCaseAssetPool.getAsset(d).getStartPrice();
        }

        if (payoffType.getType() == PayOffType.MIN_TYPE) {
            java.util.Arrays.sort(s_current);
            // the strike price is the minimum of the spot prices of the
            // underlyings
            strikePrice = s_current[0];

        } else if (payoffType.getType() == PayOffType.MAX_TYPE) {
            java.util.Arrays.sort(s_current);
            // strike price is the maximum of the spot prices of the underlyings
            strikePrice = s_current[testCaseAssetPool.getAssetCount() - 1];

        } else if (payoffType.getType() == PayOffType.AVG_TYPE) {
            // strike price is the average of the spot prices of the underlyings
            for (int d = 0; d < s_current.length; d++) {
                strikePrice += s_current[d];
            }
            strikePrice = strikePrice / (double) s_current.length;
        }
        return strikePrice;
    }

    /**
     * Simulates a random variable lognormally. It does not take correlation
     * factor into considerations.
     *
     * @param underlyingPrice  the current/spot price of an asset
     * @param interestFreeRate the interest rate which is a risk free rate
     * @param volatilityRate   the volatility of the price of the asset
     * @param dividend         the dividend paid by the stock
     * @param t                delta t, the time step between two time intervals
     * @param random_normal    a normally distributed number with mean zero and variance one
     * @return
     */
    public static double simulateLognormal(double underlyingPrice, double interestFreeRate,
            double volatilityRate, double dividend, double t, double random_normal) {
        return underlyingPrice *
            Math.exp(((interestFreeRate - dividend) - volatilityRate * volatilityRate * 0.5) * t +
                volatilityRate * Math.sqrt(t) * random_normal);
    }

    /**
     * Simulates a random variable lognormally while taking correlation factor
     * into consideration.
     *
     * @param underlyingPrice    the current/spot price of an asset
     * @param interestFreeRate   the interest rate/ risk free rate
     * @param volatilityRate     the volatility rate of the price of the asset
     * @param dividend           the dividend paid by the asset
     * @param t                  delta t, the time step between two time intervals
     * @param correlation_square square of the correlation
     * @param correlation        the correlation factor
     * @param random_normal      a normally distributed number with mean zero and variance one
     * @return
     */
    public static double simulateLognormal(double underlyingPrice, double interestFreeRate,
            double volatilityRate, double dividend, double t, double correlation_square, double correlation,
            double random_normal) {
        return underlyingPrice *
            Math.exp(((interestFreeRate - dividend) - volatilityRate * volatilityRate * 0.5 *
                correlation_square) *
                t + correlation * volatilityRate * Math.sqrt(t) * random_normal);
    }

}
