/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.util;

import org.exolab.castor.xml.Unmarshaller;
import plugtest.data.AssetPool;
import plugtest.data.CorrelationMatrix;
import plugtest.data.TestCases;

import java.io.File;
import java.io.FileReader;


/**
 * Loads the data from the descriptor files.
 *
 * @author The ProActive Team
 */
public class LoadData {

    //@snippet-start load_test_cases_file
    /**
     * Reads the testcases from the input file.
     *
     * @param testCaseFile the file containing the testcases.
     * @return testCases returns the testcases
     */

    public static TestCases loadTestCases(File testCaseFile)
    //@snippet-end load_test_cases_file
    {

        TestCases testCases = null;

        if (!testCaseFile.exists()) {
            System.err.println(" " + testCaseFile + " does not exist.");
            System.exit(-1);
        } else if (!testCaseFile.canRead()) {
            System.err.println(" " + testCaseFile + " can not be read.");
        } else if (!testCaseFile.isFile()) {
            System.err.println("" + testCaseFile + " is not a regular file.");
        }

        try {
            FileReader testCaseFR = new FileReader(testCaseFile);
            testCases = (TestCases) Unmarshaller.unmarshal(TestCases.class, testCaseFR);
        } catch (Exception ex) {
            System.out
                    .println("Failed to load OptionSet from file: [" + testCaseFile + "]" + ex.getMessage());
            System.exit(-1);
        }

        return testCases;
    }

    //@snippet-start load_assetpool_file
    /**
     * Reads the asset pool from the input file. Asset pool is a collection of
     * assets that the testcases are based on.
     *
     * @param assetPoolFile the input file containing the asset pool
     * @return the asset pool
     */

    public static AssetPool loadAssetPool(File assetPoolFile)
    //@snippet-end load_assetpool_file
    {

        AssetPool assetPool = null;

        if (!assetPoolFile.exists()) {
            System.err.println(" " + assetPoolFile + " does not exist.");
            System.exit(-1);
        } else if (!assetPoolFile.canRead()) {
            System.err.println(" " + assetPoolFile + " can not be read.");
        } else if (!assetPoolFile.isFile()) {
            System.err.println("" + assetPoolFile + " is not a regular file.");
        }

        try {
            FileReader assetPoolFR = new FileReader(assetPoolFile);
            assetPool = (AssetPool) Unmarshaller.unmarshal(AssetPool.class, assetPoolFR);
        } catch (Exception ex) {
            System.out.println("Failed to load OptionSet from file: [" + assetPoolFile + "]" +
                ex.getMessage());
            System.exit(-1);
        }
        return assetPool;
    }

    //@snippet-start load_correlation_file
    /**
     * Reads the correlation matrix from the input file. The correlation matrix
     * is a matrix containing correlation values among the assets in the asset
     * pool.
     *
     * @param corrFile the file containing the correlation matrix
     * @return returns the correlation matrix
     */
    public static CorrelationMatrix loadCorrMatrix(File corrFile)
    //@snippet-end load_correlation_file
    {
        CorrelationMatrix corrMatrix = null;

        if (!corrFile.exists()) {
            System.err.println(" " + corrFile + " does not exist.");
            System.exit(-1);
        } else if (!corrFile.canRead()) {
            System.err.println(" " + corrFile + " can not be read.");
        } else if (!corrFile.isFile()) {
            System.err.println("" + corrFile + " is not a regular file.");
        }

        try {
            FileReader corrFR = new FileReader(corrFile);
            corrMatrix = (CorrelationMatrix) Unmarshaller.unmarshal(CorrelationMatrix.class, corrFR);
        } catch (Exception ex) {
            System.out.println("Failed to load OptionSet from file: [" + corrFile + "]" + ex.getMessage());
            System.exit(-1);
        }
        return corrMatrix;
    }

}
