/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.util;

import Jama.LUDecomposition;
import Jama.Matrix;


/**
 * Defines matrix utilities such as Cholesky decomposition of a correlation
 * matrix.
 *
 * @author The ProActive Team
 */

public class MatrixUtil {

    /**
     * To compute the Cholesky decomposition of the correlation matrix. The
     * Cholesky decomposition is a matrix decomposition of a symmetric
     * positive-definite matrix into a lower triangular matrix and the transpose
     * of the lower triangular matrix. The lower triangular matrix is the
     * Cholesky triangle of the original, positive-definite matrix.
     *
     * @param assetCount the number assets in the current test case asset pool
     * @param coMatrix   the correlation matrix of the assets
     * @return the Cholesky triangle
     */

    public static Matrix decomposition(int assetCount, Matrix coMatrix) {

        Matrix corrMatrix = null;
        boolean positive = true;
        Matrix lowerTriangularMatrix;

        if (assetCount != 1) {
            lowerTriangularMatrix = new Matrix(assetCount, assetCount, 0.0);
        } else {
            lowerTriangularMatrix = new Matrix(assetCount, assetCount, 1.0);
        }

        double sum1 = 0;
        double sum2 = 0;
        double[][] correlationMatrix = new double[assetCount][assetCount];

        // initial the matrix correlationMatrix and lowerTriangularMatrix
        for (int i = 0; i < assetCount; i++) {
            for (int j = 0; j < assetCount; j++) {
                if (i == j) {
                    correlationMatrix[i][j] = 1.0;
                } else {
                    correlationMatrix[i][j] = coMatrix.get(i, j);
                }
            }
        }

        // Verify the Sylvester criterion on the correlation matrix
        // to be sure it is possitive definite (using Sylvester's criterion)
        for (int n = 0; n < assetCount; n++) {
            corrMatrix = new Matrix(n + 1, n + 1, 0.0);
            for (int i = 0; i < n; i++) {
                for (int j = 0; j < n; j++) {
                    if (i == j) {
                        corrMatrix.set(i, j, 1.0);
                    } else {
                        corrMatrix.set(i, j, coMatrix.get(i, j));
                    }
                }
            }
            LUDecomposition LU = new LUDecomposition(corrMatrix);
            if (LU.det() < 0) {
                positive = false;
            }
            // System.out.println("Positive correlation matrix verification is "
            // + positive);
        }

        if (positive) {
            // calculate decomposition Cholesky
            for (int i = 0; i < assetCount; i++) {
                sum1 = 0;
                for (int j = 0; j < i; j++) {
                    sum1 += (lowerTriangularMatrix.get(i, j) * lowerTriangularMatrix.get(i, j));
                }
                lowerTriangularMatrix.set(i, i, (Math.sqrt(correlationMatrix[i][i] - sum1)));
                for (int j = i + 1; j < assetCount; j++) {
                    sum2 = 0;
                    for (int k = 0; k < i; k++) {
                        sum2 += (lowerTriangularMatrix.get(i, k) * lowerTriangularMatrix.get(i, k));
                    }

                    lowerTriangularMatrix.set(j, i,
                            (((correlationMatrix[i][j] - sum2) / lowerTriangularMatrix.get(i, i))));
                }
            }
            return lowerTriangularMatrix;
        } else {
            return null;
        }
    }

}
