/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.util;

import plugtest.data.types.OptionType;
import plugtest.data.types.PayOffType;

import java.io.Serializable;


/**
 * The payoff class computes the payoff for the current set of assets.
 *
 * @author The ProActive Team
 */
public class PayOff implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * The type of the payoff to be computed on a current set of spot prices
     */
    private PayOffType payoffType;

    /**
     * A set of spot of prices of a basket
     */
    private double[] s_current; // the array is could be sorted
    private double pre_payoff;

    /**
     * The computed value of payoff
     */
    private double payoff;

    /**
     * Strike price for the basket
     */

    private double strike;

    /**
     * The type of the option which the basket stands for
     */
    private OptionType optionType;

    public PayOff() {
        this.Reset();
    }

    /**
     * Results payoff and pre_payoff
     */
    public void Reset() {
        payoff = 0.0;
        pre_payoff = 0.0;
    }

    /**
     * Sets up the payoff object with necessary parameters like spot prices of
     * underlyings, strike price of the basket, etc,
     *
     * @param optionType type of the option (call or put)
     * @param payoffType type of the payoff (minimum, maximum or average)
     * @param s_current  current set of spot prices
     * @param strike     strike price of the basket
     */

    public void setup(OptionType optionType, PayOffType payoffType, double[] s_current, double strike) {
        this.optionType = optionType;
        this.payoffType = payoffType;
        this.s_current = s_current;
        this.strike = strike;
    }

    /**
     * Calculate the payoff for a basket of assets and store the payoff in
     * "payoff"
     */
    public void calculate() {
        // computes pre-payoff
        switch (this.payoffType.getType()) {
            case PayOffType.MIN_TYPE: // minimum of spot prices of underlying
                // assets
                this.pre_payoff = this.s_current[0];
                break;
            case PayOffType.MAX_TYPE:// maximum of spot prices of underlying
                // assets
                this.pre_payoff = this.s_current[this.s_current.length - 1];
                break;
            case PayOffType.AVG_TYPE:// average of spot prices of underlying
                // assets
                this.pre_payoff = 0;
                for (int d = 0; d < this.s_current.length; d++) {
                    this.pre_payoff += this.s_current[d];
                }
                this.pre_payoff = this.pre_payoff / this.s_current.length;
                break;
        }

        if (optionType.getType() == OptionType.PUT_TYPE) {

            /*
             * Payoff/Put = maximum(x - S_T, 0) (if the strike price at the
             * exercise date is greater than the market stock price, a put
             * option makes its holder profit of (x - S_T), zero otherwise)
             */
            this.payoff = Math.max(this.strike - this.pre_payoff, 0);

        } else if (optionType.getType() == OptionType.CALL_TYPE) {

            /*
             * Payoff/Call = minimum(S_T-x, 0) (if the market spot price at the
             * exercise date is greater than the strike price, a call option
             * makes its holder a profit of (S_T - x), zero otherwise)
             */

            this.payoff = Math.max(this.pre_payoff - this.strike, 0);
        }
    }

    public double getPayoff() {
        return payoff;
    }

    public void setPayoff(double payoff) {
        this.payoff = payoff;
    }

    public double getPre_payoff() {
        return pre_payoff;
    }

    public void setPre_payoff(double pre_payoff) {
        this.pre_payoff = pre_payoff;
    }

}
