/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.util;

import Jama.Matrix;
import plugtest.data.AssetPool;
import plugtest.data.CorrelationMatrix;


/**
 * This is a generalized class to process the correlation matrix and the asset
 * pool. For each test case, the basket is chosen from the given large asset
 * pool and so is the case with correlation matrix. So for each testcase, a
 * subset of assetpool and a subset of correlation matrix is chosen from the
 * "input" (superset) asset pool and correlation matrix respectively.
 *
 * @author The ProActive Team
 */
public class ProcessData {

    //@snippet-start parse_correlation_descriptor
    /**
     * Parses the correlation matrix from the CorrelationMatrix format into
     * Matrix format. This preprocessing is convenient for reducing the
     * computational time required for computing the subset of the correlation
     * matrix.
     *
     * @param corrMatrix Correlation Matrix that is defined in the data format
     * @return
     */

    public static Matrix parseCorrelationDescriptor(CorrelationMatrix corrMatrix)
    //@snippet-end parse_correlation_descriptor
    {
        int assetCount = corrMatrix.getCorrelationCount();
        Matrix mat = new Matrix(assetCount, assetCount, 0.0);
        double corrValues[] = new double[assetCount];
        for (int i = 0; i < assetCount; i++) {
            corrValues = corrMatrix.getCorrelation(i).getValues();
            for (int j = 0; j < assetCount; j++) {
                mat.set(i, j, corrValues[j]);
            }
        }
        return mat;
    }

    //@snippet-start extract_assetPool_basket
    /**
     * @param assetPool asset pool contains a set of assets
     * @param basket    the basket indices of assets which will be pulled from the
     *                  assetpool to form the subset of the assetpool for the current
     *                  test case
     * @return the extracted sub set of asset pool
     */

    public static AssetPool extractAssetPool(AssetPool assetPool, int[] basket)
    //@snippet-end extract_assetPool_basket
    {
        AssetPool testCaseAssetPool = new AssetPool();
        int basketSize = basket.length;
        for (int i = 0; i < basketSize; i++) {
            testCaseAssetPool.addAsset(assetPool.getAsset(basket[i]));
        }
        return testCaseAssetPool;
    }

    //@snippet-start extract_correlation_basket
    /**
     * @param corrMatrix a super matrix of correlation values among all the assets
     * @param basket     the basket indices of assets of which the correlation values
     *                   will be extracted from the (super) correlation matrix
     * @return a sub-correlation matrix for the current test case
     */

    public static Matrix extractCorrelationMatrix(Matrix corrMatrix, int[] basket)
    //@snippet-end extract_correlation_basket
    {
        int basketSize = basket.length;
        Matrix matrix = new Matrix(basketSize, basketSize, 0.0);
        for (int i = 0; i < basketSize; i++) {
            for (int j = 0; j < basketSize; j++) {
                matrix.set(i, j, corrMatrix.get(basket[i], basket[j]));
            }
        }
        return matrix;
    }

}
