/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.util;

import plugtest.data.Result;
import plugtest.data.types.OptionType;

import java.io.Serializable;


/**
 * Provide utility functions for processing results
 *
 * @author The ProActive Team
 */

public class ResultHelper implements Serializable {

    private static final long serialVersionUID = 1L;

    Result result;

    public ResultHelper() {
        result = null;
    }

    //@snippet-start results_update
    /**
     * Merges results from different workers
     *
     * @param partialResult partial result from a task
     */
    public synchronized void update(final Result partialResult) {
        if (result == null) {
            result = partialResult;
        } else {
            double sumX = result.getPrice().getCallPrice() + partialResult.getPrice().getCallPrice();
            double sumY = result.getPrice().getPutPrice() + partialResult.getPrice().getPutPrice();
            double variance = result.getPrice().getVariance() + partialResult.getPrice().getVariance();
            result.getPrice().setCallPrice(sumX);
            result.getPrice().setPutPrice(sumY);
            result.getPrice().setVariance(variance);
        }
    }

    //@snippet-end results_update

    //@snippet-start results_finalizePrices
    /**
     * Computes the final results
     * Refer to : http://mathworld.wolfram.com/CentralLimitTheorem.html
     * 
     * @param numberTasks the number of bags of tasks
     * @param mcIter      the number of monte carlo simulations per bag of task
     */
    public synchronized void finalizePrices(int numberTasks, long mcIter, OptionType optionType) {
        // by central limit theorem

        if (optionType.getType() == OptionType.CALL_TYPE) {
            result.getPrice().setCallPrice(result.getPrice().getCallPrice() / (numberTasks * mcIter));
            result.getPrice().setVariance(
                    result.getPrice().getVariance() / (numberTasks * mcIter) -
                        Math.pow(result.getPrice().getCallPrice(), 2));
        } else {
            result.getPrice().setPutPrice(result.getPrice().getPutPrice() / (numberTasks * mcIter));
            result.getPrice().setVariance(
                    (result.getPrice().getVariance() / (numberTasks * mcIter)) -
                        Math.pow(result.getPrice().getPutPrice(), 2));
        }
        //@snippet-end results_finalizePrices
        /*
         * confidence interval (CI) is an interval estimate of a population
         * parameter. Instead of estimating the parameter by a single value, an
         * interval likely to include the parameter is given. Thus, confidence
         * intervals are used to indicate the reliability of an estimate. How
         * likely the interval is to contain the parameter is determined by the
         * confidence level or confidence coefficient. Increasing the desired
         * confidence level will widen the confidence interval.
         * 
         * For example, a CI can be used to describe how reliable survey results
         * are. In a poll of election voting-intentions, the result might be
         * that 40% of respondents intend to vote for a certain party. A 95%
         * confidence interval for the proportion in the whole population having
         * the same intention on the survey date might be 36% to 44%. All other
         * things being equal, a survey result with a small CI is more reliable
         * than a result with a large CI and one of the main things controlling
         * this width in the case of population surveys is the size of the
         * sample questioned. Confidence intervals and interval estimates more
         * generally have applications across the whole range of quantitative
         * studies.
         */
        result.getPrice().setCI(1.95 * Math.sqrt(result.getPrice().getVariance() / (numberTasks * mcIter)));

        // You can add the code for greeks and metrics computing
    }

    public Result getResult() {
        return result;
    }

    public void setResult(final Result result) {
        this.result = result;
    }

}
