/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.test;

import Jama.Matrix;
import org.apache.commons.cli.*;
import org.apache.log4j.Logger;
import org.apache.log4j.Level;
import org.objectweb.proactive.core.ProActiveException;
import org.objectweb.proactive.core.util.log.Loggers;
import org.objectweb.proactive.extensions.masterworker.TaskException;
import plugtest.data.*;
import plugtest.pricing.engines.MCEuropeanPricingEngine;
import plugtest.pricing.option.MCEuropeanBasketOption;
import plugtest.pricing.util.DisplayData;
import plugtest.pricing.util.LoadData;
import plugtest.pricing.util.ProcessData;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.net.MalformedURLException;


/**
 * This test class is an example of how to process the input and out descriptors
 * for the GridPlugtests2008 - Super Quant Monte Carlo Option Pricing challenge.
 *
 * @author The ProActive Team
 */
public class GridPlugtestsV2008 {

    public static final String DEFAULT_DEPLOY_DESCRIPTOR = "GCMA.xml";
    public static final String DEFAULT_CORRMATRIX_DESCRIPTOR = "testcases/correlation0.xml";
    public static final String DEFAULT_ASSETPOOL_DESCRIPTOR = "testcases/assetPool0.xml";
    public static final String DEFAULT_TESTCASES_DESCRIPTOR = "testcases/testCases0.xml";

    /**
     * Participants MUST use this name-format for the storing their final results
     */
    public static final String FINAL_RESULTS_DESCRIPTOR = "testcases/GridPlugtestsV2008Results-TEAMNAME.xml";

    static File testCasesDescriptorFile;
    static File assetPoolDescriptorFile;
    static File correlationMatrixDescriptorFile;
    static File deploymentDescriptorFile;

    static String vn_name;
    static String master_vn_name;

    public static void main(String[] args) throws ProActiveException, TaskException, FileNotFoundException,
            MalformedURLException {

        init(args);

        Matrix testCaseCorrMatrix;
        AssetPool testCaseAssetPool;
        Results results = new Results();
        Metrics metrics = new Metrics();

        //@snippet-start main_example
        // load the data from testcases descriptor file
        final TestCases testCases = LoadData.loadTestCases(testCasesDescriptorFile);

        // load the asset pool from the assetpool descriptor file
        final AssetPool assetPool = LoadData.loadAssetPool(assetPoolDescriptorFile);

        // load the correlation matrix from the correlation matrix descriptor file
        final CorrelationMatrix corrMatrix = LoadData.loadCorrMatrix(correlationMatrixDescriptorFile);

        // get the fixed parameters from the test cases
        final BasketParams basketParams = testCases.getBasketParams();

        // get the correlation descriptor in a jama.Matrix format
        final Matrix matCorrelation = ProcessData.parseCorrelationDescriptor(corrMatrix);

        /**
         * Create a monte carlo based european option pricing engine
         * it will price an euroean/barrier option using monte carlo based methods.
         * The workers and master's virtual names can be given as arguments otherwise are assigned
         * default values.
         * Since engine deploys the application in the grid setting, we have to provide
         * the GCM deployment descriptor file to it.
         */
        MCEuropeanPricingEngine mcEngine = new MCEuropeanPricingEngine(deploymentDescriptorFile, vn_name,
            master_vn_name);

        long startTime = System.currentTimeMillis();

        MCEuropeanBasketOption mcEuroBsk = new MCEuropeanBasketOption(basketParams);
        // Now, lets set the monte carlo based pricing engine to the basket     
        mcEuroBsk.setPricingEngine(mcEngine);

        // initiate the Monte Carlo API and hence the ProActive runtime
        mcEuroBsk.initMC();

        // process each test case
        for (int i = 0; i < testCases.getTestCaseCount(); i++) {

            /**
             * We first need to construct the basket option out of the global asset pool.
             * As the testcase includes the indices of the assets from this global pool,
             * we must extract the assets from it before processing it.
             */
            testCaseAssetPool = ProcessData.extractAssetPool(assetPool, testCases.getTestCase(i).getBasket());

            /**
             * We also need to extract submatrix from the global correlation matrix.
             * The same basket of indices is used to get this submatrix.
             */
            testCaseCorrMatrix = ProcessData.extractCorrelationMatrix(matCorrelation, testCases
                    .getTestCase(i).getBasket());
            /**
             * With all the paramters, market constants such as interest rate, time per interal and monte carlo
             * simulations in basketParams, the correlation matrix for the sub set of assets,
             * other parameters associated with testcase such as the execution type, option type, pay off types and time
             * to maturity and, of course, the assets in the basket,
             * we construct the baskey option using the MCEuropeanBasketOption constructor.
             */
            mcEuroBsk.set(testCaseCorrMatrix, testCases.getTestCase(i), testCaseAssetPool);

            // Fire the engine to price the option using monte carlo based algorithm
            mcEuroBsk.calculateMC();

            // Get the result for this testcase and update the final results
            results.addResult(mcEuroBsk.getResult());
        }

        long stopTime = System.currentTimeMillis();
        metrics.setStartTime(startTime);
        metrics.setStopTime(stopTime);
        metrics.setWallTime((stopTime - startTime) / 1000);

        // update the metrics for final results.
        results.setMetrics(metrics); //

        // store the final results in the out put descriptor file
        DisplayData.printResults(results, new FileOutputStream(FINAL_RESULTS_DESCRIPTOR));
        //@snippet-end main_example

        // NOTE : The participant MUST stick to format of the final results file
        System.out.println("The final results are written to " + FINAL_RESULTS_DESCRIPTOR);

        mcEuroBsk.terminateMC();

    }

    /**
     * Parses the command line arguments and sets the appropriate file
     * descriptors.
     *
     * @param args command line arguments
     * @throws MalformedURLException
     */
    public static void init(String[] args) throws MalformedURLException {

        findOS();

        Options command_options = new Options();
        command_options.addOption(OptionBuilder.withArgName("file").hasArg().withDescription(
                "GCM application descriptor").create("d"));
        command_options.addOption("w", true, "Workers virtual node name");
        command_options.addOption("m", true, "master virtual node name");
        command_options.addOption(OptionBuilder.withArgName("file").hasArg().withDescription(
                "Assetpool descriptor").create("a"));
        command_options.addOption(OptionBuilder.withArgName("file").hasArg().withDescription(
                "Correlation matrix descriptor").create("c"));
        command_options.addOption(OptionBuilder.withArgName("file").hasArg().withDescription(
                "Test cases descriptor").create("t"));
        command_options.addOption(new Option("debug", "print debugging information"));

        // automatically generate the help statement
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp("GridPlugtestsV2008", command_options);

        CommandLineParser parser = new GnuParser();

        CommandLine cmd = null;

        try {
            cmd = parser.parse(command_options, args);
        } catch (ParseException e) {
            System.err.println("Parsing failed, reason, " + e.getMessage());
            System.exit(1);
        }

        // get descriptor option value
        String descPath = cmd.getOptionValue("d");

        if (descPath == null) {
            deploymentDescriptorFile = new File(DEFAULT_DEPLOY_DESCRIPTOR);
            if (deploymentDescriptorFile == null) {
                System.err.println("Couldn't find internal ressource: " + DEFAULT_DEPLOY_DESCRIPTOR);
                System.exit(1);
            }
        } else {
            // check provided descriptor
            deploymentDescriptorFile = new File(descPath);
            validateFile(deploymentDescriptorFile);
        }

        // get vn option value
        vn_name = cmd.getOptionValue("w");
        master_vn_name = cmd.getOptionValue("m");

        descPath = cmd.getOptionValue("a");

        if (descPath == null) {
            assetPoolDescriptorFile = new File(DEFAULT_ASSETPOOL_DESCRIPTOR);
            if (assetPoolDescriptorFile == null) {
                System.err.println("Couldn't find internal ressource: " + DEFAULT_ASSETPOOL_DESCRIPTOR);
                System.exit(1);
            }
        } else {
            // check provided descriptor
            File assetPoolDescriptorFile = new File(descPath);
            validateFile(assetPoolDescriptorFile);
        }

        // get the correlation matrix descriptor
        descPath = cmd.getOptionValue("c");

        if (descPath == null) {
            correlationMatrixDescriptorFile = new File(DEFAULT_CORRMATRIX_DESCRIPTOR);
            if (correlationMatrixDescriptorFile == null) {
                System.err.println("Couldn't find internal ressource: " + DEFAULT_CORRMATRIX_DESCRIPTOR);
                System.exit(1);
            }
        } else {
            // check provided descriptor
            File correlationMatrixDescriptorFile = new File(descPath);
            validateFile(correlationMatrixDescriptorFile);
        }

        descPath = cmd.getOptionValue("t");

        if (descPath == null) {
            testCasesDescriptorFile = new File(DEFAULT_TESTCASES_DESCRIPTOR);
            if (testCasesDescriptorFile == null) {
                System.err.println("Couldn't find internal ressource: " + DEFAULT_TESTCASES_DESCRIPTOR);
                System.exit(1);
            }
        } else {
            // check provided descriptor
            File testCasesDescriptorFile = new File(descPath);
            validateFile(testCasesDescriptorFile);
        }
        if (cmd.hasOption("debug")) {
            Logger logger = Logger.getLogger(Loggers.MASTERWORKER);
            logger.setLevel(Level.DEBUG);
        }

    }

    public static void validateFile(File file) {
        // validate if the file description is correct argument
        if (!file.exists()) {
            System.err.println("" + file + " does not exist");
            System.exit(1);
        } else if (!file.canRead()) {
            System.err.println("" + file + " can't be read");
            System.exit(1);
        } else if (!file.isFile()) {
            System.err.println("" + file + " is not a regular file");
            System.exit(1);
        }

    }

    public static void findOS() {
        // Finding current os
        String osName = System.getProperty("os.name");
        if (osName.toLowerCase().contains("windows")) {
            System.setProperty("os", "windows");
        } else {
            System.setProperty("os", "unix");
        }
    }
}
